@@ -97,14 +97,28 @@ class ADBase:
     #
 
     def get_ad_api(self):
-        api = adapi.ADAPI(self.AD, self.name, self._logging, self.args, self.config, self.app_config, self.global_vars,)
+        api = adapi.ADAPI(
+            self.AD,
+            self.name,
+            self._logging,
+            self.args,
+            self.config,
+            self.app_config,
+            self.global_vars,
+        )
 
         return api
 
     @utils.sync_wrapper
     async def get_plugin_api(self, plugin_name):
         return await self.AD.plugins.get_plugin_api(
-            plugin_name, self.name, self._logging, self.args, self.config, self.app_config, self.global_vars,
+            plugin_name,
+            self.name,
+            self._logging,
+            self.args,
+            self.config,
+            self.app_config,
+            self.global_vars,
         )
 
     #
