@@ -66,7 +66,6 @@ class AppManagement:
         self.AD.services.register_service("admin", "app", "create", self.manage_services)
         self.AD.services.register_service("admin", "app", "edit", self.manage_services)
         self.AD.services.register_service("admin", "app", "remove", self.manage_services)
-        self.AD.services.register_service("admin", "app", "reload", self.manage_services)
 
         self.active_apps = []
         self.inactive_apps = []
@@ -214,7 +213,8 @@ class AppManagement:
                 error_logger.warning("-" * 60)
                 if self.AD.logging.separate_error_log() is True:
                     self.logger.warning(
-                        "Logged an error to %s", self.AD.logging.get_filename("error_log"),
+                        "Logged an error to %s",
+                        self.AD.logging.get_filename("error_log"),
                     )
 
                 executed = False
@@ -223,11 +223,12 @@ class AppManagement:
             if name in self.objects:
                 del self.objects[name]
 
-            if name in self.global_module_dependencies:
-                del self.global_module_dependencies[name]
+            # if name in self.global_module_dependencies:
+            #    del self.global_module_dependencies[name]
 
         else:
-            self.objects[name]["running"] = False
+            if name in self.objects:
+                self.objects[name]["running"] = False
 
         await self.increase_inactive_apps(name)
 
@@ -265,10 +266,13 @@ class AppManagement:
         else:
             await self.initialize_app(app)
 
-    async def stop_app(self, app, delete=True):
+    async def stop_app(self, app, delete=False):
         executed = False
         try:
-            self.logger.info("Terminating %s", app)
+            if "global" in self.app_config[app] and self.app_config[app]["global"] is True:
+                pass
+            else:
+                self.logger.info("Terminating %s", app)
             executed = await self.terminate_app(app, delete)
         except Exception:
             error_logger = logging.getLogger("Error.{}".format(app))
@@ -296,7 +300,10 @@ class AppManagement:
 
         app_args = self.app_config[name]
         self.logger.info(
-            "Initializing app %s using class %s from module %s", name, app_args["class"], app_args["module"],
+            "Initializing app %s using class %s from module %s",
+            name,
+            app_args["class"],
+            app_args["module"],
         )
 
         if self.get_file_from_module(app_args["module"]) is not None:
@@ -334,7 +341,13 @@ class AppManagement:
                 self.objects[name] = {
                     "type": "app",
                     "object": app_class(
-                        self.AD, name, self.AD.logging, app_args, self.AD.config, self.app_config, self.AD.global_vars,
+                        self.AD,
+                        name,
+                        self.AD.logging,
+                        app_args,
+                        self.AD.config,
+                        self.app_config,
+                        self.AD.global_vars,
                     ),
                     "id": uuid.uuid4().hex,
                     "pin_app": self.AD.threading.app_should_be_pinned(name),
@@ -348,7 +361,9 @@ class AppManagement:
 
         else:
             self.logger.warning(
-                "Unable to find module module %s - '%s' is not initialized", app_args["module"], name,
+                "Unable to find module module %s - '%s' is not initialized",
+                app_args["module"],
+                name,
             )
             await self.increase_inactive_apps(name)
 
@@ -410,6 +425,9 @@ class AppManagement:
                                     if config[app] is not None:
                                         app_valid = True
                                         if app == "global_modules":
+                                            self.logger.warning(
+                                                f"global_modules directive has been deprecated and will be removed in a future release"
+                                            )
                                             #
                                             # Check the parameter format for string or list
                                             #
@@ -442,11 +460,20 @@ class AppManagement:
                                         ):
                                             valid_apps[app] = config[app]
                                             valid_apps[app]["yaml_path"] = path
+                                        elif (
+                                            isinstance(config[app], dict)
+                                            and "module" in config[app]
+                                            and "global" in config[app]
+                                            and config[app]["global"] is True
+                                        ):
+                                            valid_apps[app] = config[app]
+                                            valid_apps[app]["yaml_path"] = path
                                         else:
                                             app_valid = False
                                             if self.AD.invalid_yaml_warnings:
                                                 self.logger.warning(
-                                                    "App '%s' missing 'class' or 'module' entry - ignoring", app,
+                                                    "App '%s' missing 'class' or 'module' entry - ignoring",
+                                                    app,
                                                 )
 
                                         if app_valid is True:
@@ -458,7 +485,8 @@ class AppManagement:
                             else:
                                 if self.AD.invalid_yaml_warnings:
                                     self.logger.warning(
-                                        "File '%s' invalid structure - ignoring", os.path.join(root, file),
+                                        "File '%s' invalid structure - ignoring",
+                                        os.path.join(root, file),
                                     )
 
                             if new_config is None:
@@ -478,7 +506,9 @@ class AppManagement:
 
                                 if app in new_config:
                                     self.logger.warning(
-                                        "File '%s' duplicate app: %s - ignoring", os.path.join(root, file), app,
+                                        "File '%s' duplicate app: %s - ignoring",
+                                        os.path.join(root, file),
+                                        app,
                                     )
                                 else:
                                     new_config[app] = valid_apps[app]
@@ -652,7 +682,7 @@ class AppManagement:
                         # Since the entry has been deleted we can't sensibly determine dependencies
                         # So just immediately terminate it
                         #
-                        await self.terminate_app(name)
+                        await self.terminate_app(name, delete=True)
                         await self.remove_entity(name)
 
                 for name in new_config:
@@ -689,7 +719,8 @@ class AppManagement:
                             if self.AD.invalid_yaml_warnings:
                                 if silent is False:
                                     self.logger.warning(
-                                        "App '%s' missing 'class' or 'module' entry - ignoring", name,
+                                        "App '%s' missing 'class' or 'module' entry - ignoring",
+                                        name,
                                     )
 
                 self.app_config = new_config
@@ -699,23 +730,20 @@ class AppManagement:
                     if name in self.app_config:
                         total_apps -= 1  # remove one
 
-                # if silent is False:
-                self.logger.info("Found %s total apps", total_apps)
+                active_apps, inactive, glbl = self.get_active_app_count()
 
+                # if silent is False:
                 await self.set_state(
-                    self.total_apps_sensor, state=total_apps, attributes={"friendly_name": "Total Apps"}
+                    self.total_apps_sensor, state=active_apps + inactive, attributes={"friendly_name": "Total Apps"}
                 )
 
-                active_apps = self.get_active_app_count()
-
-                inactive_apps = total_apps - active_apps
-                if inactive_apps > 0:
-                    self.logger.info("Found %s active apps", active_apps)
-                    self.logger.info("Found %s inactive apps", inactive_apps)
+                self.logger.info("Found %s active apps", active_apps)
+                self.logger.info("Found %s inactive apps", inactive)
+                self.logger.info("Found %s global libraries", glbl)
 
             # Now we know if we have any new apps we can create new threads if pinning
 
-            active_apps = self.get_active_app_count()
+            active_apps, inactive, glbl = self.get_active_app_count()
 
             if add_threads is True and self.AD.threading.auto_pin is True:
                 if active_apps > self.AD.threading.thread_count:
@@ -736,15 +764,19 @@ class AppManagement:
             self.logger.warning("-" * 60)
 
     def get_active_app_count(self):
-        c = 0
+        active = 0
+        inactive = 0
+        glbl = 0
         for name in self.app_config:
             if "disable" in self.app_config[name] and self.app_config[name]["disable"] is True:
-                pass
+                inactive += 1
+            elif "global" in self.app_config[name] and self.app_config[name]["global"] is True:
+                glbl += 1
             elif name in self.non_apps:
                 pass
             else:
-                c += 1
-        return c
+                active += 1
+        return active, inactive, glbl
 
     def get_app_from_file(self, file):
         module = self.get_module_from_path(file)
@@ -779,16 +811,22 @@ class AppManagement:
         else:
             app = self.get_app_from_file(file)
             if app is not None:
-                self.logger.info("Loading App Module: %s", file)
-                if module_name not in self.modules:
+                if "global" in self.app_config[app] and self.app_config[app]["global"] is True:
+                    # It's a new style global module
+                    self.logger.info("Loading Global Module: %s", file)
                     self.modules[module_name] = importlib.import_module(module_name)
                 else:
-                    # We previously imported it so we need to reload to pick up any potential changes
-                    importlib.reload(self.modules[module_name])
-
+                    # A regular app
+                    self.logger.info("Loading App Module: %s", file)
+                    if module_name not in self.modules:
+                        self.modules[module_name] = importlib.import_module(module_name)
+                    else:
+                        # We previously imported it so we need to reload to pick up any potential changes
+                        importlib.reload(self.modules[module_name])
             elif "global_modules" in self.app_config and module_name in self.app_config["global_modules"]:
                 self.logger.info("Loading Global Module: %s", file)
                 self.modules[module_name] = importlib.import_module(module_name)
+            # elif "global" in
             else:
                 if self.AD.missing_app_warnings:
                     self.logger.warning("No app description found for: %s - ignoring", file)
@@ -944,13 +982,12 @@ class AppManagement:
                         apps["term"][app] = 1
                     apps["init"][app] = 1
 
-                if "global_modules" in self.app_config:
-                    for gm in utils.single_or_list(self.app_config["global_modules"]):
-                        if gm == self.get_module_from_path(module["name"]):
-                            for app in self.apps_per_global_module(gm):
-                                if module["reload"]:
-                                    apps["term"][app] = 1
-                                apps["init"][app] = 1
+                for gm in self.get_global_modules():
+                    if gm == self.get_module_from_path(module["name"]):
+                        for app in self.apps_per_global_module(gm):
+                            if module["reload"]:
+                                apps["term"][app] = 1
+                            apps["init"][app] = 1
 
             if plugin is not None:
                 self.logger.info("Processing restart for %s", plugin)
@@ -982,6 +1019,18 @@ class AppManagement:
             if apps is not None and apps["term"]:
                 prio_apps = self.get_app_deps_and_prios(apps["term"], mode)
 
+                # Mark dependant global modules for reload
+                for app in sorted(prio_apps, key=prio_apps.get):
+                    found = False
+                    for module in modules:
+                        if module["name"] == self.get_path_from_app(app):
+                            found = True
+                            module["reload"] = True
+                    if found is False:
+                        if self.get_path_from_app(app) is not None:
+                            modules.append({"name": self.get_path_from_app(app), "reload": True})
+
+                # Terminate Apps
                 for app in sorted(prio_apps, key=prio_apps.get, reverse=True):
                     executed = await self.stop_app(app)
                     apps_terminated[app] = executed
@@ -1021,6 +1070,9 @@ class AppManagement:
                             self.logger.info("%s is disabled", app)
                             await self.set_state(app, state="disabled")
                             await self.increase_inactive_apps(app)
+                        elif "global" in self.app_config[app] and self.app_config[app]["global"] is True:
+                            await self.set_state(app, state="global")
+                            await self.increase_inactive_apps(app)
                         else:
                             if apps_terminated.get(app, True) is True:  # the app terminated properly
                                 await self.init_object(app)
@@ -1037,7 +1089,8 @@ class AppManagement:
                         error_logger.warning("-" * 60)
                         if self.AD.logging.separate_error_log() is True:
                             self.logger.warning(
-                                "Logged an error to %s", self.AD.logging.get_filename("error_log"),
+                                "Logged an error to %s",
+                                self.AD.logging.get_filename("error_log"),
                             )
 
                 await self.AD.threading.calculate_pin_threads()
@@ -1047,6 +1100,8 @@ class AppManagement:
                 for app in sorted(prio_apps, key=prio_apps.get):
                     if "disable" in self.app_config[app] and self.app_config[app]["disable"] is True:
                         pass
+                    elif "global" in self.app_config[app] and self.app_config[app]["global"] is True:
+                        pass
                     else:
                         if apps_terminated.get(app, True) is True:  # the app terminated properly
                             await self.initialize_app(app)
@@ -1065,6 +1120,10 @@ class AppManagement:
 
             self.apps_initialized = True
 
+    def get_path_from_app(self, app):
+        module = self.app_config[app]["module"]
+        return self.get_file_from_module(module)
+
     def get_app_deps_and_prios(self, applist, mode):
 
         # Build a list of modules and their dependencies
@@ -1082,20 +1141,23 @@ class AppManagement:
 
         for app in full_list:
             dependees = []
-            if "dependencies" in self.app_config[app]:
-                for dep in utils.single_or_list(self.app_config[app]["dependencies"]):
-                    if dep in self.app_config:
-                        dependees.append(dep)
-                    else:
-                        self.logger.warning("Unable to find app %s in dependencies for %s", dep, app)
-                        self.logger.warning("Ignoring app %s", app)
+            for dep in self.get_app_dependencies(app):
+                if dep in self.app_config:
+                    dependees.append(dep)
+                else:
+                    self.logger.warning("Unable to find app %s in dependencies for %s", dep, app)
+                    self.logger.warning("Ignoring app %s", app)
             deps.append((app, dependees))
 
         prio_apps = {}
         prio = float(50.1)
         try:
             for app in self.topological_sort(deps):
-                if "dependencies" in self.app_config[app] or self.app_has_dependents(app):
+                if (
+                    "dependencies" in self.app_config[app]
+                    or app in self.global_module_dependencies
+                    or self.app_has_dependents(app)
+                ):
                     prio_apps[app] = prio
                     prio += float(0.0001)
                 else:
@@ -1117,22 +1179,20 @@ class AppManagement:
 
     def app_has_dependents(self, name):
         for app in self.app_config:
-            if "dependencies" in self.app_config[app]:
-                for dep in utils.single_or_list(self.app_config[app]["dependencies"]):
-                    if dep == name:
-                        return True
+            for dep in self.get_app_dependencies(app):
+                if dep == name:
+                    return True
         return False
 
     def get_dependent_apps(self, dependee, deps):
         for app in self.app_config:
-            if "dependencies" in self.app_config[app]:
-                for dep in utils.single_or_list(self.app_config[app]["dependencies"]):
-                    # print("app= {} dep = {}, dependee = {} deps = {}".format(app, dep, dependee, deps))
-                    if dep == dependee and app not in deps:
-                        deps.append(app)
-                        new_deps = self.get_dependent_apps(app, deps)
-                        if new_deps is not None:
-                            deps.append(new_deps)
+            for dep in self.get_app_dependencies(app):
+                # print("app= {} dep = {}, dependee = {} deps = {}".format(app, dep, dependee, deps))
+                if dep == dependee and app not in deps:
+                    deps.append(app)
+                    new_deps = self.get_dependent_apps(app, deps)
+                    if new_deps is not None:
+                        deps.append(new_deps)
 
     def topological_sort(self, source):
 
@@ -1179,13 +1239,25 @@ class AppManagement:
                     if gm == module:
                         apps.append(app)
 
-        for app, gms in self.global_module_dependencies.items():
-            for gm in gms:
-                if gm == module:
-                    apps.append(app)
+            if "dependencies" in self.app_config[app]:
+                for gm in utils.single_or_list(self.app_config[app]["dependencies"]):
+                    if gm == module:
+                        apps.append(app)
 
         return apps
 
+    def get_app_dependencies(self, app):
+        deps = []
+        if "dependencies" in self.app_config[app]:
+            for dep in utils.single_or_list(self.app_config[app]["dependencies"]):
+                deps.append(dep)
+
+        if app in self.global_module_dependencies:
+            for dep in self.global_module_dependencies[app]:
+                deps.append(dep)
+
+        return deps
+
     def create_app(self, app=None, **kwargs):
         """Used to create an app, which is written to a Yaml file"""
 
@@ -1372,21 +1444,37 @@ class AppManagement:
             elif isinstance(module, object) and module.__class__.__name__ == "module":
                 module_name = module.__name__
 
-            if (
-                module_name is not None
-                and "global_modules" in self.app_config
-                and module_name in self.app_config["global_modules"]
-            ):
+            if module_name is not None:
+                if (
+                    "global_modules" in self.app_config and module_name in self.app_config["global_modules"]
+                ) or self.is_global_module(module_name):
 
-                if name not in self.global_module_dependencies:
-                    self.global_module_dependencies[name] = []
+                    if name not in self.global_module_dependencies:
+                        self.global_module_dependencies[name] = []
 
-                if module_name not in self.global_module_dependencies[name]:
-                    self.global_module_dependencies[name].append(module_name)
-            else:
-                self.logger.warning(
-                    "Module %s not in global_modules in register_module_dependency() for %s", module_name, name,
-                )
+                    if module_name not in self.global_module_dependencies[name]:
+                        self.global_module_dependencies[name].append(module_name)
+                else:
+                    self.logger.warning(
+                        "Module %s not a global_modules in register_module_dependency() for %s",
+                        module_name,
+                        name,
+                    )
+
+    def get_global_modules(self):
+        gms = []
+        if "global_modules" in self.app_config:
+            for gm in utils.single_or_list(self.app_config["global_modules"]):
+                gms.append(gm)
+
+        for app in self.app_config:
+            if "global" in self.app_config[app] and self.app_config[app]["global"] is True:
+                gms.append(self.app_config[app]["module"])
+
+        return gms
+
+    def is_global_module(self, module):
+        return module in self.get_global_modules()
 
     async def manage_services(self, namespace, domain, service, kwargs):
         app = kwargs.pop("app", None)
