@@ -24,6 +24,7 @@ class Scheduler:
         self.last_fired = None
         self.sleep_task = None
         self.active = False
+        self.timer_resetted = False
         self.location = None
         self.schedule = {}
 
@@ -91,19 +92,169 @@ class Scheduler:
         self.logger.debug("stop() called for scheduler")
         self.stopping = True
 
-    async def cancel_timer(self, name, handle):
+    async def insert_schedule(self, name, aware_dt, callback, repeat, type_, **kwargs):
+
+        # aware_dt will include a timezone of some sort - convert to utc timezone
+        utc = aware_dt.astimezone(pytz.utc)
+
+        # we get the time now
+        now = await self.get_now()
+
+        # Round to nearest second
+        #
+        # Take this out to allow fractional run_in() times
+        #
+        # utc = self.my_dt_round(utc, base=1)
+
+        if "pin" in kwargs:
+            pin_app = kwargs["pin"]
+        else:
+            pin_app = self.AD.app_management.objects[name]["pin_app"]
+
+        if "pin_thread" in kwargs:
+            pin_thread = kwargs["pin_thread"]
+            pin_app = True
+        else:
+            pin_thread = self.AD.app_management.objects[name]["pin_thread"]
+
+        if name not in self.schedule:
+            self.schedule[name] = {}
+
+        handle = uuid.uuid4().hex
+        c_offset = self.get_offset({"kwargs": kwargs})
+        ts = utc + timedelta(seconds=c_offset)
+        interval = kwargs.get("interval", 0)
+        basetime_interval = (ts - now).seconds
+
+        self.schedule[name][handle] = {
+            "name": name,
+            "id": self.AD.app_management.objects[name]["id"],
+            "callback": callback,
+            "timestamp": ts,
+            "interval": interval,
+            "basetime": utc,
+            "basetime_interval": basetime_interval,
+            "repeat": repeat,
+            "offset": c_offset,
+            "type": type_,
+            "pin_app": pin_app,
+            "pin_thread": pin_thread,
+            "kwargs": kwargs,
+        }
+
+        if callback is None:
+            function_name = "cancel_callback"
+        else:
+            function_name = callback.__name__
+
+        await self.AD.state.add_entity(
+            "admin",
+            "scheduler_callback.{}".format(handle),
+            "active",
+            {
+                "app": name,
+                "execution_time": utils.dt_to_str(ts.replace(microsecond=0), self.AD.tz),
+                "repeat": str(datetime.timedelta(seconds=interval)),
+                "function": function_name,
+                "pinned": pin_app,
+                "pinned_thread": pin_thread,
+                "fired": 0,
+                "executed": 0,
+                "kwargs": kwargs,
+            },
+        )
+        # verbose_log(conf.logger, "INFO", conf.schedule[name][handle])
+
+        if self.active is True:
+            await self.kick()
+
+        return handle
+
+    async def cancel_timer(self, name, handle, silent):
         executed = False
         self.logger.debug("Canceling timer for %s", name)
         if self.timer_running(name, handle):
             del self.schedule[name][handle]
-            await self.AD.state.remove_entity("admin", "scheduler_callback.{}".format(handle))
+            await self.AD.state.remove_entity("admin", f"scheduler_callback.{handle}")
             executed = True
 
         if name in self.schedule and self.schedule[name] == {}:
             del self.schedule[name]
 
+        if not executed and silent is False:
+            self.logger.warning(f"Invalid callback handle '{handle}' in cancel_timer() from app {name}")
+
+        return executed
+
+    async def restart_timer(self, uuid_: str, args: dict, restart_offset: int = 0) -> dict:
+        """Used to restart a timer"""
+
+        if args["type"] == "next_rising" or args["type"] == "next_setting":
+            c_offset = self.get_offset(args)
+            args["timestamp"] = self.sun(args["type"], c_offset)
+            args["offset"] = c_offset
+
+        else:
+            # Not sunrise or sunset so just increment
+            # the timestamp with the repeat interval
+            if restart_offset > 0:
+                # we to restart with an offset
+                new_timestamp = args["timestamp"] + timedelta(seconds=restart_offset)
+                args["timestamp"] = new_timestamp
+
+            else:
+                args["basetime"] += timedelta(seconds=args["interval"])
+                args["timestamp"] = args["basetime"] + timedelta(seconds=self.get_offset(args))
+
+        # Update entity
+
+        await self.AD.state.set_state(
+            "_scheduler",
+            "admin",
+            f"scheduler_callback.{uuid_}",
+            execution_time=utils.dt_to_str(args["timestamp"].replace(microsecond=0), self.AD.tz),
+        )
+
+        return args
+
+    async def reset_timer(self, name: str, handle: str) -> bool:
+        """Used to reset a timer"""
+
+        executed = False
+
+        if self.timer_running(name, handle):
+            self.logger.debug("Resetting timer %s for %s", handle, name)
+
+            args = await utils.run_in_executor(self, utils.deepcopy, self.schedule[name][handle])
+
+            if args["type"] == "next_rising" or args["type"] == "next_setting":
+                self.logger.warning(
+                    f"The given handle '{handle}' in reset_timer() from app {name} is a Sun timer, cannot reset that"
+                )
+                return executed
+
+            # we get the time now
+            now = await self.get_now()
+
+            # we get the time from now to be added
+            basetime_interval = args["basetime_interval"]
+            restart_offset = basetime_interval - (args["timestamp"] - now).seconds
+
+            args = await self.restart_timer(handle, args, restart_offset)
+            self.schedule[name][handle] = args
+
+            if self.active is True:
+                await self.kick()
+
+            executed = True
+
+            # we need to indicate a reset took place
+            self.timer_resetted = True
+
         if not executed:
-            self.logger.warning("Invalid callback handle '{}' in cancel_timer() from app {}".format(handle, name))
+            self.logger.warning(
+                f"The given handle '{handle}' in reset_timer() from app {name}, doesn't have a running timer"
+            )
 
         return executed
 
@@ -155,7 +306,9 @@ class Scheduler:
                         if "__timeout" in args["kwargs"] and self.timer_running(
                             name, args["kwargs"]["__timeout"]
                         ):  # meaning there is a timeout for this callback
-                            await self.cancel_timer(name, args["kwargs"]["__timeout"])  # cancel it as no more needed
+                            await self.cancel_timer(
+                                name, args["kwargs"]["__timeout"], False
+                            )  # cancel it as no more needed
 
             elif "__state_handle" in args["kwargs"]:
                 #
@@ -191,23 +344,9 @@ class Scheduler:
                 )
             # If it is a repeating entry, rewrite with new timestamp
             if args["repeat"]:
-                if args["type"] == "next_rising" or args["type"] == "next_setting":
-                    c_offset = self.get_offset(args)
-                    args["timestamp"] = self.sun(args["type"], c_offset)
-                    args["offset"] = c_offset
-                else:
-                    # Not sunrise or sunset so just increment
-                    # the timestamp with the repeat interval
-                    args["basetime"] += timedelta(seconds=args["interval"])
-                    args["timestamp"] = args["basetime"] + timedelta(seconds=self.get_offset(args))
-                # Update entity
+                # restart the timer
+                args = await self.restart_timer(uuid_, args)
 
-                await self.AD.state.set_state(
-                    "_scheduler",
-                    "admin",
-                    "scheduler_callback.{}".format(uuid_),
-                    execution_time=utils.dt_to_str(args["timestamp"].replace(microsecond=0), self.AD.tz),
-                )
             else:
                 # Otherwise just delete
                 await self.AD.state.remove_entity("admin", "scheduler_callback.{}".format(uuid_))
@@ -250,6 +389,12 @@ class Scheduler:
         else:
             return self.next_sunset(day_offset)
 
+    def todays_sunrise(self, days_offset):
+        candidate_date = (self.now + datetime.timedelta(days=days_offset)).astimezone(self.AD.tz).date()
+        next_rising_dt = self.location.sunrise(date=candidate_date, local=False, observer_elevation=self.AD.elevation)
+
+        return next_rising_dt
+
     def next_sunrise(self, offset: int = 0):
         day_offset = 0
         while True:
@@ -282,6 +427,12 @@ class Scheduler:
 
         return next_setting_dt
 
+    def todays_sunset(self, days_offset):
+        candidate_date = (self.now + datetime.timedelta(days=days_offset)).astimezone(self.AD.tz).date()
+        next_setting_dt = self.location.sunset(date=candidate_date, local=False, observer_elevation=self.AD.elevation)
+
+        return next_setting_dt
+
     @staticmethod
     def get_offset(kwargs: dict):
         if "offset" in kwargs["kwargs"]:
@@ -299,75 +450,6 @@ class Scheduler:
             # self.logger.debug("get_offset(): offset = %s", offset)
         return offset
 
-    async def insert_schedule(self, name, aware_dt, callback, repeat, type_, **kwargs):
-
-        # aware_dt will include a timezone of some sort - convert to utc timezone
-        utc = aware_dt.astimezone(pytz.utc)
-
-        # Round to nearest second
-
-        utc = self.my_dt_round(utc, base=1)
-
-        if "pin" in kwargs:
-            pin_app = kwargs["pin"]
-        else:
-            pin_app = self.AD.app_management.objects[name]["pin_app"]
-
-        if "pin_thread" in kwargs:
-            pin_thread = kwargs["pin_thread"]
-            pin_app = True
-        else:
-            pin_thread = self.AD.app_management.objects[name]["pin_thread"]
-
-        if name not in self.schedule:
-            self.schedule[name] = {}
-        handle = uuid.uuid4().hex
-        c_offset = self.get_offset({"kwargs": kwargs})
-        ts = utc + timedelta(seconds=c_offset)
-        interval = kwargs.get("interval", 0)
-
-        self.schedule[name][handle] = {
-            "name": name,
-            "id": self.AD.app_management.objects[name]["id"],
-            "callback": callback,
-            "timestamp": ts,
-            "interval": interval,
-            "basetime": utc,
-            "repeat": repeat,
-            "offset": c_offset,
-            "type": type_,
-            "pin_app": pin_app,
-            "pin_thread": pin_thread,
-            "kwargs": kwargs,
-        }
-
-        if callback is None:
-            function_name = "cancel_callback"
-        else:
-            function_name = callback.__name__
-
-        await self.AD.state.add_entity(
-            "admin",
-            "scheduler_callback.{}".format(handle),
-            "active",
-            {
-                "app": name,
-                "execution_time": utils.dt_to_str(ts.replace(microsecond=0), self.AD.tz),
-                "repeat": str(datetime.timedelta(seconds=interval)),
-                "function": function_name,
-                "pinned": pin_app,
-                "pinned_thread": pin_thread,
-                "fired": 0,
-                "executed": 0,
-                "kwargs": kwargs,
-            },
-        )
-        # verbose_log(conf.logger, "INFO", conf.schedule[name][handle])
-
-        if self.active is True:
-            await self.kick()
-        return handle
-
     async def terminate_app(self, name):
         if name in self.schedule:
             for id in self.schedule[name]:
@@ -473,10 +555,12 @@ class Scheduler:
                 now = pytz.utc.localize(datetime.datetime.utcnow())
                 if self.realtime is True:
                     self.now = now
+
                 else:
                     if result is True:
                         # We got kicked so lets figure out the elapsed pseudo time
                         delta = (now - self.last_fired).total_seconds() * self.AD.timewarp
+
                     else:
                         if len(next_entries) > 0:
                             # Time is progressing infinitely fast and it's already time for our next callback
@@ -510,6 +594,11 @@ class Scheduler:
                     #
                     next_entries = self.get_next_entries()
 
+                elif self.timer_resetted is True:
+                    # a timer was resetted, so need to recalculate next entries
+                    next_entries = self.get_next_entries()
+                    self.timer_resetted = False
+
                 old_dst_offset = dst_offset
                 #
                 # OK, lets fire the entries
@@ -585,7 +674,7 @@ class Scheduler:
 
     async def sleep(self, delay):
         coro = asyncio.sleep(delay)
-        self.sleep_task = asyncio.ensure_future(coro)
+        self.sleep_task = asyncio.create_task(coro)
         try:
             await self.sleep_task
             self.sleep_task = None
@@ -617,14 +706,17 @@ class Scheduler:
                 self.sanitize_timer_kwargs(self.AD.app_management.objects[name]["object"], callback["kwargs"]),
             )
         else:
-            self.logger.warning("Invalid timer handle given as: %s", handle)
+            # self.logger.warning("Invalid timer handle given as: %s", handle)
             return None
 
     async def get_scheduler_entries(self):
         schedule = {}
         for name in self.schedule.keys():
             schedule[name] = {}
-            for entry in sorted(self.schedule[name].keys(), key=lambda uuid_: self.schedule[name][uuid_]["timestamp"],):
+            for entry in sorted(
+                self.schedule[name].keys(),
+                key=lambda uuid_: self.schedule[name][uuid_]["timestamp"],
+            ):
                 schedule[name][str(entry)] = {}
                 schedule[name][str(entry)]["timestamp"] = str(
                     self.AD.sched.make_naive(self.schedule[name][entry]["timestamp"])
@@ -695,49 +787,92 @@ class Scheduler:
     async def get_now_naive(self):
         return self.make_naive(await self.get_now())
 
-    async def now_is_between(self, start_time_str, end_time_str, name=None):
-        start_time = (await self._parse_time(start_time_str, name))["datetime"]
-        end_time = (await self._parse_time(end_time_str, name))["datetime"]
-        now = (await self.get_now()).astimezone(self.AD.tz)
-        start_date = now.replace(hour=start_time.hour, minute=start_time.minute, second=start_time.second)
-        end_date = now.replace(hour=end_time.hour, minute=end_time.minute, second=end_time.second)
-        if end_date < start_date:
+    async def now_is_between(self, start_time_str, end_time_str, name=None, now=None):
+        start_time = (await self._parse_time(start_time_str, name, today=True, days_offset=0))["datetime"]
+        end_time = (await self._parse_time(end_time_str, name, today=True, days_offset=0))["datetime"]
+        if now is not None:
+            now = (await self._parse_time(now, name))["datetime"]
+        else:
+            now = (await self.get_now()).astimezone(self.AD.tz)
+
+        # self.diag.info(f"locals: {locals()}")
+        # Comparisons
+        if end_time < start_time:
+            # self.diag.info("Midnight transition")
+            # Start and end time backwards.
             # Spans midnight
-            if now < start_date and now < end_date:
-                now = now + datetime.timedelta(days=1)
-            end_date = end_date + datetime.timedelta(days=1)
-        return start_date <= now <= end_date
+            # Lets start by assuming end_time is wrong and should be tomorrow
+            # This will be true if we are currently after start_time
+            end_time = (await self._parse_time(end_time_str, name, today=True, days_offset=1))["datetime"]
+            if now < start_time and now < end_time:
+                # self.diag.info("Reverse")
+                # Well, it's complicated -
+                # We crossed into a new day and things changed.
+                # Now all times have shifted relative to the new day, so we need to look at it differently
+                # If both times are now in the future, we now actually want to set start time back a day and keep end_time as today
+                start_time = (await self._parse_time(start_time_str, name, today=True, days_offset=-1))["datetime"]
+                end_time = (await self._parse_time(end_time_str, name, today=True, days_offset=0))["datetime"]
 
-    async def sunset(self, aware):
+        # self.diag.info(f"\nstart = {start_time}\nnow   = {now}\nend   = {end_time}")
+        return start_time <= now <= end_time
+
+    async def sunset(self, aware, today=False, days_offset=0):
         if aware is True:
-            return self.next_sunset().astimezone(self.AD.tz)
+            if today is True:
+                return self.todays_sunset(days_offset).astimezone(self.AD.tz)
+            else:
+                return self.next_sunset().astimezone(self.AD.tz)
         else:
-            return self.make_naive(self.next_sunset().astimezone(self.AD.tz))
+            if today is True:
+                return self.make_naive(self.todays_sunset(days_offset).astimezone(self.AD.tz))
+            else:
+                return self.make_naive(self.next_sunset().astimezone(self.AD.tz))
 
-    async def sunrise(self, aware):
+    async def sunrise(self, aware, today=False, days_offset=0):
         if aware is True:
-            return self.next_sunrise().astimezone(self.AD.tz)
+            if today is True:
+                return self.todays_sunrise(days_offset).astimezone(self.AD.tz)
+            else:
+                return self.next_sunrise().astimezone(self.AD.tz)
         else:
-            return self.make_naive(self.next_sunrise().astimezone(self.AD.tz))
+            if today is True:
+                return self.make_naive(self.todays_sunrise(days_offset).astimezone(self.AD.tz))
+            else:
+                return self.make_naive(self.next_sunrise().astimezone(self.AD.tz))
 
-    async def parse_time(self, time_str, name=None, aware=False):
+    async def parse_time(self, time_str, name=False, aware=False, today=False, days_offset=0):
         if aware is True:
-            return (await self._parse_time(time_str, name))["datetime"].astimezone(self.AD.tz).time()
+            return (
+                (await self._parse_time(time_str, name, today=today, days_offset=days_offset))["datetime"]
+                .astimezone(self.AD.tz)
+                .time()
+            )
         else:
-            return self.make_naive((await self._parse_time(time_str, name))["datetime"]).time()
+            return self.make_naive(
+                (await self._parse_time(time_str, name, today=today, days_offset=days_offset))["datetime"]
+            ).time()
 
-    async def parse_datetime(self, time_str, name=None, aware=False):
+    async def parse_datetime(self, time_str, name=None, aware=False, today=False, days_offset=0):
         if aware is True:
-            return (await self._parse_time(time_str, name))["datetime"].astimezone(self.AD.tz)
+            return (await self._parse_time(time_str, name, today=today, days_offset=days_offset))[
+                "datetime"
+            ].astimezone(self.AD.tz)
         else:
-            return self.make_naive((await self._parse_time(time_str, name))["datetime"])
+            return self.make_naive(
+                (await self._parse_time(time_str, name, today=today, days_offset=days_offset))["datetime"]
+            )
 
-    async def _parse_time(self, time_str, name=None):
+    async def _parse_time(self, time_str, name=None, today=False, days_offset=0):
         parsed_time = None
         sun = None
         offset = 0
-        parts = re.search(r"^(\d+)-(\d+)-(\d+)\s+(\d+):(\d+):(\d+)$", time_str)
+        parts = re.search(r"^(\d+)-(\d+)-(\d+)\s+(\d+):(\d+):(\d+)(?:\.(\d+))?$", time_str)
         if parts:
+            if parts.group(7) is None:
+                us = 0
+            else:
+                us = int(float("0." + parts.group(7)) * 1000000)
+
             this_time = datetime.datetime(
                 int(parts.group(1)),
                 int(parts.group(2)),
@@ -745,57 +880,78 @@ class Scheduler:
                 int(parts.group(4)),
                 int(parts.group(5)),
                 int(parts.group(6)),
-                0,
+                us,
             )
-            parsed_time = self.AD.tz.localize(this_time)
+            parsed_time = self.AD.tz.localize(this_time + datetime.timedelta(days=days_offset))
         else:
-            parts = re.search(r"^(\d+):(\d+):(\d+)$", time_str)
+            parts = re.search(r"^(\d+):(\d+):(\d+)(?:\.(\d+))?$", time_str)
             if parts:
-                today = (await self.get_now()).astimezone(self.AD.tz)
-                time = datetime.time(int(parts.group(1)), int(parts.group(2)), int(parts.group(3)), 0)
-                parsed_time = today.replace(hour=time.hour, minute=time.minute, second=time.second, microsecond=0,)
+                if parts.group(4) is None:
+                    us = 0
+                else:
+                    us = int(float("0." + parts.group(4)) * 1000000)
 
+                today = (await self.get_now()).astimezone(self.AD.tz)
+                time = datetime.time(int(parts.group(1)), int(parts.group(2)), int(parts.group(3)), us)
+                parsed_time = today.replace(
+                    hour=time.hour,
+                    minute=time.minute,
+                    second=time.second,
+                    microsecond=us,
+                ) + datetime.timedelta(days=days_offset)
             else:
                 if time_str == "sunrise":
-                    parsed_time = await self.sunrise(True)
+                    parsed_time = await self.sunrise(True, today, days_offset)
                     sun = "sunrise"
                     offset = 0
                 elif time_str == "sunset":
-                    parsed_time = await self.sunset(True)
+                    parsed_time = await self.sunset(True, today, days_offset)
                     sun = "sunset"
                     offset = 0
                 else:
-                    parts = re.search(r"^sunrise\s*([+-])\s*(\d+):(\d+):(\d+)$", time_str)
+                    parts = re.search(r"^sunrise\s*([+-])\s*(\d+):(\d+):(\d+)(?:\.(\d+))?$", time_str)
                     if parts:
+                        if parts.group(5) is None:
+                            us = 0
+                        else:
+                            us = int(float("0." + parts.group(5)) * 1000000)
+
                         sun = "sunrise"
+                        td = datetime.timedelta(
+                            hours=int(parts.group(2)),
+                            minutes=int(parts.group(3)),
+                            seconds=int(parts.group(4)),
+                            microseconds=us,
+                        )
+
                         if parts.group(1) == "+":
-                            td = datetime.timedelta(
-                                hours=int(parts.group(2)), minutes=int(parts.group(3)), seconds=int(parts.group(4)),
-                            )
                             offset = td.total_seconds()
-                            parsed_time = await self.sunrise(True) + td
+                            parsed_time = await self.sunrise(True, today, days_offset) + td
                         else:
-                            td = datetime.timedelta(
-                                hours=int(parts.group(2)), minutes=int(parts.group(3)), seconds=int(parts.group(4)),
-                            )
                             offset = td.total_seconds() * -1
-                            parsed_time = await self.sunrise(True) - td
+                            parsed_time = await self.sunrise(True, today, days_offset) - td
                     else:
-                        parts = re.search(r"^sunset\s*([+-])\s*(\d+):(\d+):(\d+)$", time_str)
+                        parts = re.search(r"^sunset\s*([+-])\s*(\d+):(\d+):(\d+)(?:\.(\d+))?$", time_str)
                         if parts:
+                            if parts.group(5) is None:
+                                us = 0
+                            else:
+                                us = int(float("0." + parts.group(5)) * 1000000)
+
                             sun = "sunset"
+                            td = datetime.timedelta(
+                                hours=int(parts.group(2)),
+                                minutes=int(parts.group(3)),
+                                seconds=int(parts.group(4)),
+                                microseconds=us,
+                            )
                             if parts.group(1) == "+":
-                                td = datetime.timedelta(
-                                    hours=int(parts.group(2)), minutes=int(parts.group(3)), seconds=int(parts.group(4)),
-                                )
                                 offset = td.total_seconds()
-                                parsed_time = await self.sunset(True) + td
+                                parsed_time = await self.sunset(True, today, days_offset) + td
                             else:
-                                td = datetime.timedelta(
-                                    hours=int(parts.group(2)), minutes=int(parts.group(3)), seconds=int(parts.group(4)),
-                                )
                                 offset = td.total_seconds() * -1
-                                parsed_time = await self.sunset(True) - td
+                                parsed_time = await self.sunset(True, today, days_offset) - td
+
         if parsed_time is None:
             if name is not None:
                 raise ValueError("%s: invalid time string: %s", name, time_str)
@@ -812,7 +968,9 @@ class Scheduler:
         self.diag.info("Sun")
         self.diag.info("--------------------------------------------------")
         self.diag.info("Next Sunrise: %s", self.next_sunrise())
+        self.diag.info("Today's Sunrise: %s", self.todays_sunrise(days_offset=0))
         self.diag.info("Next Sunset: %s", self.next_sunset())
+        self.diag.info("Today's Sunset: %s", self.todays_sunset(days_offset=0))
         self.diag.info("--------------------------------------------------")
 
     async def dump_schedule(self):
@@ -825,7 +983,8 @@ class Scheduler:
             for name in self.schedule.keys():
                 self.diag.info("%s:", name)
                 for entry in sorted(
-                    self.schedule[name].keys(), key=lambda uuid_: self.schedule[name][uuid_]["timestamp"],
+                    self.schedule[name].keys(),
+                    key=lambda uuid_: self.schedule[name][uuid_]["timestamp"],
                 ):
                     self.diag.info(
                         " Next Event Time: %s - data: %s",
@@ -878,5 +1037,11 @@ class Scheduler:
     def make_naive(self, dt):
         local = dt.astimezone(self.AD.tz)
         return datetime.datetime(
-            local.year, local.month, local.day, local.hour, local.minute, local.second, local.microsecond,
+            local.year,
+            local.month,
+            local.day,
+            local.hour,
+            local.minute,
+            local.second,
+            local.microsecond,
         )
