@@ -196,7 +196,12 @@ class State:
                 exec_time = await self.AD.sched.get_now() + datetime.timedelta(seconds=int(timeout))
 
                 kwargs["__timeout"] = await self.AD.sched.insert_schedule(
-                    name, exec_time, None, False, None, __state_handle=handle,
+                    name,
+                    exec_time,
+                    None,
+                    False,
+                    None,
+                    __state_handle=handle,
                 )
             #
             # In the case of a quick_start parameter,
@@ -448,9 +453,14 @@ class State:
             # We assume that the event will come back to us via the plugin
             await plugin.remove_entity(namespace, entity)
 
-        if entity in self.state[namespace]:
-            self.state[namespace].pop(entity)
-            data = {"event_type": "__AD_ENTITY_REMOVED", "data": {"entity_id": entity}}
+        await self.remove_entity_simple(namespace, entity)
+
+    async def remove_entity_simple(self, namespace: str, entity_id: str) -> None:
+        """Used to remove an internal AD entity"""
+
+        if entity_id in self.state[namespace]:
+            self.state[namespace].pop(entity_id)
+            data = {"event_type": "__AD_ENTITY_REMOVED", "data": {"entity_id": entity_id}}
             self.AD.loop.create_task(self.AD.events.process_event(namespace, data))
 
     async def add_entity(self, namespace, entity, state, attributes=None):
@@ -477,6 +487,15 @@ class State:
 
         self.AD.loop.create_task(self.AD.events.process_event(namespace, data))
 
+    def get_state_simple(self, namespace, entity_id):
+        # Simple sync version of get_state() primarily for use in entity objects, returns whole state for the entity
+        if namespace not in self.state:
+            raise ValueError(f"Namespace {namespace} not found for entity.state")
+        if entity_id not in self.state[namespace]:
+            raise ValueError(f"Entity {entity_id} not found in namespace {namespace} for entity.state")
+
+        return self.state[namespace][entity_id]
+
     async def get_state(self, name, namespace, entity_id=None, attribute=None, default=None, copy=True):
         self.logger.debug("get_state: %s.%s %s %s", entity_id, attribute, default, copy)
 
